<?php

/**
 * @file
 * Contains Drush commands for CMS Content Sync.
 */

use Drupal\cms_content_sync\Entity\Pool;
use Drupal\cms_content_sync\Entity\Flow;
use Drupal\cms_content_sync\SyncCorePoolExport;
use Drupal\cms_content_sync\SyncCoreFlowExport;

/**
 * Implements hook_drush_command().
 */
function cms_content_sync_drush_command() {
  $items['content-sync-export'] = [
    'description' => dt('Export configuration to the Sync Core.'),
    'aliases' => ['cse'],
  ];

  $items['content-sync-login'] = [
    'description' => dt('Export configuration to the Sync Core.'),
    'aliases' => ['csl'],
  ];

  return $items;
}

/**
 * Export configuration to the Content Sync backend.
 *
 * @options force Whether to ignore that another site is already using the
 *     same site ID. Useful if you change the URL of a site.
 *
 * @throws \Exception
 */
function drush_cms_content_sync_content_sync_export($options = ['force' => FALSE]) {
  drush_print('Started export of pools.');
  foreach (Pool::getAll() as $pool) {
    $exporter = new SyncCorePoolExport($pool);

    if (!$options['force'] && $exporter->siteIdExists()) {
      throw new \Exception('Another site is already using this site ID for the pool ' . $pool->id);
    }

    $steps = $exporter->prepareBatch();
    foreach ($steps as $step) {
      $exporter->executeBatch($step);
    }
  }
  drush_print('Finished export of pools.');

  drush_print('Started export of flows.');
  foreach (Flow::getAll() as $flow) {
    $exporter = new SyncCoreFlowExport($flow);
    $steps    = $exporter->prepareBatch();
    foreach ($steps as $step) {
      $exporter->executeBatch($step);
    }
  }
  drush_print('Finished export of flows.');
}

/**
 * Kindly ask the Sync Core to login again.
 */
function drush_cms_content_sync_content_sync_login() {
  drush_print('Calling /login for all connections.');

  $result = [];
  foreach (Flow::getAll() as $flow) {
    $exporter = new SyncCoreFlowExport($flow);
    $result = array_merge($result, $exporter->login());
  }

  foreach ($result as $url => $success) {
    if (!$success) {
      $url = preg_replace('%https?://(.*)@(.*)$%', '$2', $url);
      drush_print('FAILED to login from ' . $url);
    }
  }

  drush_print('Finished login.');
}
