<?php

namespace Drupal\cms_content_sync\Commands;

use Drupal\cms_content_sync\Entity\Flow;
use Drupal\cms_content_sync\Entity\Pool;
use Drupal\cms_content_sync\SyncCoreFlowExport;
use Drupal\cms_content_sync\SyncCorePoolExport;
use Drush\Commands\DrushCommands;

/**
 * CMS Content Sync Drush Commands.
 */
class CMSContentSyncCommands extends DrushCommands {

  /**
   * Export configuration to the Sync Core.
   *
   * @command cms_content_sync:content-sync-export
   * @aliases cse
   * @options force Whether to ignore that another site is already using the
   *     same site ID. Useful if you change the URL of a site.
   *
   * @throws \Exception
   */
  public function content_sync_export($options = ['force' => FALSE]) {
    $this->output()->writeln('Started export of pools.');
    foreach (Pool::getAll() as $pool) {
      $exporter = new SyncCorePoolExport($pool);

      if (!$options['force'] && $exporter->siteIdExists()) {
        throw new \Exception('Another site is already using the site ID "' . $pool->getSiteId() . '" for the pool "' . $pool->id . '"');
      }

      $steps = $exporter->prepareBatch();
      foreach ($steps as $step) {
        $exporter->executeBatch($step);
      }
    }
    $this->output()->writeln('Finished export of pools.');
    $this->output()->writeln('Started export of flows.');
    foreach (Flow::getAll() as $flow) {
      $exporter = new SyncCoreFlowExport($flow);
      $steps    = $exporter->prepareBatch();
      foreach ($steps as $step) {
        $exporter->executeBatch($step);
      }
    }
    $this->output()->writeln('Finished export of flows.');
  }

  /**
   * Kindly ask the Sync Core to login again.
   *
   * @command cms_content_sync:content-sync-login
   * @aliases csl
   */
  public function content_sync_login() {
    $this->output()->writeln('Calling /login for all connections.');

    $result = [];
    foreach (Flow::getAll() as $flow) {
      $exporter = new SyncCoreFlowExport($flow);
      $result = array_merge($result, $exporter->login());
    }

    foreach ($result as $url => $success) {
      if (!$success) {
        $url = preg_replace('%https?://(.*)@(.*)$%', '$2', $url);
        $this->output()->writeln('FAILED to login from ' . $url);
      }
    }

    $this->output()->writeln('Finished login.');
  }

}
